package com.mortals.xhx.base.system.gentable.service;


import com.mortals.framework.model.Context;
import com.mortals.framework.service.ICRUDService;
import com.mortals.xhx.base.system.gentable.model.GentableEntity;
import com.mortals.xhx.base.system.gentable.model.GentableQuery;
import com.mortals.xhx.base.system.gentable.model.ProjectInfoEntity;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;
import java.util.Map;

/**
 * <p>Title: 代码生成业务表</p>
 * <p>Description: GentableService service接口 </p>
 * <p>Copyright: Copyright &reg;  </p>
 *
 * @version 1.0.0
 */

public interface GentableService extends ICRUDService<GentableEntity, Long> {

    /**
     * 查询据库列表
     *
     * @param query 业务信息
     * @return 数据库表集合
     */
    List<GentableEntity> selectDbTableList(GentableQuery query);


    /**
     * 查询据库列表
     *
     * @param tableNames 表名称组
     * @return 数据库表集合
     */
    List<GentableEntity> selectDbTableListByNames(String[] tableNames);

    /**
     * 查询所有表信息
     *
     * @return 表信息集合
     */
    List<GentableEntity> selectGenTableAll(Long appId);

    /**
     * 导入表结构
     *
     * @param tableList 导入表列表
     */
    void importGenTable(List<GentableEntity> tableList,Long appId, Context context);


    /**
     * 预览代码
     *
     * @param id 表Id
     * @return 预览数据列表
     */
    Map<String, String> previewCode(Long id);

    /**
     * 生成代码（单表生成下载方式）
     *
     * @param tableId 表tableId
     * @return 数据
     */
    byte[] downloadCode(Long tableId,String genTypes);

    /**
     * 批量生成代码（下载方式）
     *
     * @param tableIds 表数组
     * @return 数据
     */
    byte[] downloadCode(Long[] tableIds,String genTypes);

    /**
     * 同步数据库
     *
     * @param tableId 表tableId
     */
    void synchDb(Long tableId);

    /**
     * 导入doc文档数据
     *
     * @param file 文件
     */
    void importDoc(MultipartFile file,Long appId, Context context);


    /**
     * 导出文档
     *
     * @param projectInfoEntity
     * @return 数据
     */
    byte[] exportDoc(ProjectInfoEntity projectInfoEntity);

    /**
     * 自动建表
     *
     * @param tableSql
     */
    void crateTableBySql(String tableSql, Context context);
}